import { DomBird } from "./types";
var INNER_PADDING = 4;
var Z_INDEX = "9999";
var ANIMATION_DURATION_CSS = ".2s";
var Bird = /** @class */ (function () {
    function Bird(name) {
        this.options = {};
        this.name = name;
        this.node = document.createElement("div");
        this.node.classList.add(DomBird.CLASS_NAME);
        // Insert a node to body but leave it invisible
        document.body.appendChild(this.node);
        return this;
    }
    Bird.prototype.flyTo = function (targetNode, options) {
        if (options === void 0) { options = {}; }
        this.targetNode = targetNode;
        this.options = options;
        // Style the box
        this.styleTheBox();
        // Clear children
        this.clearChildren();
        if (options.scrollIntoView) {
            // if targetNode is not present in the current view,
            // scroll to the target first.
            targetNode.scrollIntoView({ block: "center" });
        }
        // Attach label if needed
        this.attachLabel();
        // Animate to the target node
        this.animate();
    };
    // Remove the node from body
    Bird.prototype.kill = function () {
        document.body.removeChild(this.node);
    };
    Bird.prototype.styleTheBox = function () {
        // Styles
        if (!this.options.noAnimation) {
            this.node.style.transition = "all " + ANIMATION_DURATION_CSS + " ease-out";
        }
        this.node.style.opacity = "0";
        if (this.options.intent) {
            this.node.style.boxShadow = DomBird.IntentColor[this.options.intent];
        }
        else {
            this.node.style.boxShadow = DomBird.IntentColor[DomBird.Intent.PRIMARY];
        }
        this.node.style.borderRadius = "4px";
        this.node.style.position = "absolute";
        this.node.style.top = "0px";
        this.node.style.left = "0px";
        this.node.style.width = window.innerWidth + "px";
        this.node.style.height = window.innerHeight + "px";
        this.node.style.zIndex = Z_INDEX;
        if (!this.options.touchable) {
            this.node.style.pointerEvents = "none";
        }
    };
    Bird.prototype.clearChildren = function () {
        while (this.node.firstChild) {
            this.node.removeChild(this.node.firstChild);
        }
    };
    Bird.prototype.attachLabel = function () {
        if (this.options.label) {
            // Attach label
            var labelDiv = document.createElement("div");
            labelDiv.textContent = this.options.label;
            labelDiv.style.position = "absolute";
            labelDiv.style.whiteSpace = "nowrap";
            labelDiv.style.left = "0px";
            if (!this.options.labelLocation || this.options.labelLocation === "top") {
                labelDiv.style.top = "0px";
                labelDiv.style.transform = "translate(0, -100%)";
            }
            else {
                labelDiv.style.bottom = "0px";
                labelDiv.style.transform = "translate(0, 100%)";
            }
            if (this.options.intent) {
                labelDiv.style.color = DomBird.IntentTextColor[this.options.intent];
            }
            else {
                labelDiv.style.color = DomBird.IntentTextColor[DomBird.Intent.PRIMARY];
            }
            if (this.options.labelFontSize) {
                labelDiv.style.fontSize = this.options.labelFontSize;
            }
            else {
                labelDiv.style.fontSize = DomBird.DEFAULT_LABEL_FONT_SIZE;
            }
            this.node.appendChild(labelDiv);
        }
    };
    Bird.prototype.animate = function () {
        var bodyScrollTop = document.body.scrollTop;
        var _a = this.targetNode.getBoundingClientRect(), width = _a.width, height = _a.height, top = _a.top, left = _a.left;
        this.node.style.top = top + bodyScrollTop - INNER_PADDING + "px";
        this.node.style.left = left - INNER_PADDING + "px";
        this.node.style.zIndex = Z_INDEX;
        this.node.style.width = width + INNER_PADDING * 2 + "px";
        this.node.style.height = height + INNER_PADDING * 2 + "px";
        this.node.style.opacity = "1";
    };
    return Bird;
}());
export default Bird;
